#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include "DHT.h"
#include <HardwareSerial.h>
#include "BluetoothSerial.h"

#define DHTPIN 4
#define DHTTYPE DHT11

BluetoothSerial SerialBT;
HardwareSerial mh_z19b(1);
byte cmd[9] = {0xFF,0x01,0x86,0x00,0x00,0x00,0x00,0x00,0x79}; 
unsigned char co2SensorResponse[9]; 
unsigned int currentCo2Value = 0;

LiquidCrystal_I2C lcd(0x27, 16, 2);
DHT dht(DHTPIN, DHTTYPE);
int gruen = 12;
int gelb = 13;
int rot = 14;
float t,told,h,hold;

void setup() {
  Serial.begin(9600);
  lcd.init();                     
  lcd.backlight();
  dht.begin();
  mh_z19b.begin(9600, SERIAL_8N1, 16, 17); 
  pinMode(gruen,OUTPUT);
  pinMode(gelb,OUTPUT);
  pinMode(rot,OUTPUT);
  SerialBT.begin("Fergers_CO2_Ampel");
  start();
}

void start(){
 
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print(" CO2-Ampel fuer");
  lcd.setCursor(0,1);
  lcd.print("Sie allein :-)");
  lcd.backlight();
  delay(1000);
  lcd.noBacklight();
  digitalWrite(gruen,HIGH);
  digitalWrite(gelb,HIGH);
  digitalWrite(rot,HIGH);
  delay(1000);
  lcd.backlight();
  digitalWrite(gruen,LOW);
  digitalWrite(gelb,LOW);
  digitalWrite(rot,LOW);
  delay(1000);
  lcd.noBacklight();
  digitalWrite(gruen,HIGH);
  digitalWrite(gelb,HIGH);
  digitalWrite(rot,HIGH);
  delay(1000);
  lcd.backlight();
  digitalWrite(gruen,LOW);
  digitalWrite(gelb,LOW);
  digitalWrite(rot,LOW);
  delay(1000);
  lcd.noBacklight();
  digitalWrite(gruen,HIGH);
  digitalWrite(gelb,HIGH);
  digitalWrite(rot,HIGH);
  delay(1000);
  lcd.backlight();
  digitalWrite(gruen,LOW);
  digitalWrite(gelb,LOW);
  digitalWrite(rot,LOW);
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("   von Ihrem");
  lcd.setCursor(0,1);
  lcd.print(" Jochen Ferger");
  lcd.noBacklight();
  digitalWrite(gruen,HIGH);
  digitalWrite(gelb,HIGH);
  digitalWrite(rot,HIGH);
  delay(1000);
  lcd.backlight();
  digitalWrite(gruen,LOW);
  digitalWrite(gelb,LOW);
  digitalWrite(rot,LOW);
  delay(1000);
  lcd.noBacklight();
  digitalWrite(gruen,HIGH);
  digitalWrite(gelb,HIGH);
  digitalWrite(rot,HIGH);
  delay(1000);
  lcd.backlight();
  digitalWrite(gruen,LOW);
  digitalWrite(gelb,LOW);
  digitalWrite(rot,LOW);
  delay(1000);
  lcd.clear();    
}

void loop() {
  readCo2SensorValue();
  int ppm = 256 * (unsigned int) co2SensorResponse[2] + (unsigned int) co2SensorResponse[3];
  Serial.print("Co2 ppm: ");
  Serial.println(ppm);
  Serial.print("Temperature = ");
  //delay(1000);
  float t = dht.readTemperature();
  float h = dht.readHumidity();
  if(isnan(t)){
  t = told;
  }else{
  told = t;  
  }
  if(isnan(h)){
  h = hold;
  }else{
  hold = h;  
  }
  lcd.setCursor(0,0);
  lcd.print("Co2: ");
  lcd.print(ppm);
  lcd.print(" PpM");
  lcd.setCursor(0,1);
  lcd.print(told);
  lcd.print("C");

  lcd.setCursor(8,1);
  lcd.print(hold);
  lcd.print("%");
  if(ppm<1000){
    digitalWrite(gruen,HIGH);
    digitalWrite(gelb,LOW);
    digitalWrite(rot,LOW); 
  }else{
    if(ppm<1500){
      digitalWrite(gruen,LOW);
      digitalWrite(gelb,HIGH);
      digitalWrite(rot,LOW);       
    }else{
      digitalWrite(gruen,LOW);
      digitalWrite(gelb,LOW);
      digitalWrite(rot,HIGH);   
    }  
  }
  
  Serial.println();
  delay(2000); 
  String btSend = "";
  btSend += String(ppm)+":"+String(t)+":"+String(h);
  SerialBT.println(btSend);
  lcd.clear();
}

void readCo2SensorValue() {
  mh_z19b.write(cmd, 9);
  memset(co2SensorResponse, 0, 9);
  if(mh_z19b.available()) {
    mh_z19b.readBytes(co2SensorResponse, 9);
  }else{
    Serial.println("Keine Daten vom MH");  
  }
}
